#####################################################################
#   ArcGIS Interface Module
#
#   This module contains classes to interface with ArcGIS and 
#   includes selected tools of the following toolboxes:
#   - Spatial Analyst Tools
#   - Conversion Tools
#   - Data Management Tools
#   - Analysis Tools
#   - ArcPy Classes
#
#   The classes provide insulation from ArcGIS changes, as well as
#   friendlier error messages to the main code executing specific 
#   tasks because each tool is enclosed in a try except block and 
#   will return information on which tool failed, as well as the 
#   error messages provided by ArcGIS and by Python.
#
#   Modified from code created by Jim Graham and posted to his
#   class website:
#   http://ibis.colostate.edu/DH.php?WC=/WS/Jim/Geo599/08_3_AnArcGISIntrfaceClass.html
#
#   Author: Colleen Sullivan
#   Date:   6/2012
#
#####################################################################
# The arcpy library is accessed and overwriting of files is enabled
import arcpy
arcpy.env.overwriteOutput=True

#####################################################################
# Class to interface with the SPATIAL ANALYST TOOLS toolbox
#####################################################################
class SAInterface:

	#############################################################
	# Constructor for the spatial analyst interface class
	#############################################################
	# Called when the class is created	
	def __init__(self):
		# Ensure the spatial analyst extension is checked out
		# to prevent errors	
		arcpy.CheckOutExtension("Spatial")

	#############################################################
	# Create constant value (0) raster (Spatial Analyst Tools -> 
	# Raster Creation -> Create Constant Raster)
	#############################################################
	def CreateRas(self,Extent):
		try:
			TheRaster=arcpy.sa.CreateConstantRaster("0","INTEGER","0.0004",Extent)
			return(TheRaster)
		except Exception, err:
			raise RuntimeError("** Error: Create Constant Raster Failed ("+str(err)+")")

	#############################################################
	# Convert NoData values to 0
	#############################################################
	def ConvertNoData(self,Input):
		try:
			from arcpy.sa import *
			TheRaster = Con(IsNull(Input),0,Input)
			return(TheRaster)
		except Exception, err:
			raise RuntimeError("** Error: Convert NoData Failed ("+str(err)+")")

	#############################################################
	# Map algebra (code modified from:
	# http://forums.arcgis.com/threads/27808-Calculate-sum-of-2334-raster-layers)
	#############################################################
	def MapAlgebra(self, TheList):
		try:
			from arcpy.sa import *
			i = 0
			for TheFile in TheList:
				if i == 0:
					TheRaster = arcpy.Raster(TheFile)
					i+=1
				else:
					TheRaster = TheRaster + arcpy.Raster(TheFile)
					i+=1
			return(TheRaster)
		except Exception, err:
			raise RuntimeError("** Error: Map Algebra Failed ("+str(err)+")")
		
	#############################################################
	# Weighted overlay addition
	#############################################################
	def WeightedOverlayAddition(self, Folder):
		try:
			from arcpy.sa import *
			TheRaster = arcpy.Raster(Folder+"cable")
			TheRaster = TheRaster + arcpy.Raster(Folder+"disposaldump")
			TheRaster = TheRaster + arcpy.Raster(Folder+"dredge")
			TheRaster = TheRaster + arcpy.Raster(Folder+"fish_closure")
			TheRaster = TheRaster + arcpy.Raster(Folder+"fish_line")
			TheRaster = TheRaster + arcpy.Raster(Folder+"fish_other")
			TheRaster = TheRaster + arcpy.Raster(Folder+"fish_pots")
			TheRaster = TheRaster + arcpy.Raster(Folder+"fish_traps")
			TheRaster = TheRaster + arcpy.Raster(Folder+"fish_trawl")
			TheRaster = TheRaster + arcpy.Raster(Folder+"fish_troll")
			TheRaster = TheRaster + arcpy.Raster(Folder+"habitat")
			TheRaster = TheRaster + arcpy.Raster(Folder+"marinetrans_1")
			TheRaster = TheRaster + arcpy.Raster(Folder+"marinetrans_2")
			TheRaster = TheRaster + arcpy.Raster(Folder+"marinetrans_3")
			TheRaster = TheRaster + arcpy.Raster(Folder+"marinetrans_4")
			TheRaster = TheRaster + arcpy.Raster(Folder+"military")
			TheRaster = TheRaster + arcpy.Raster(Folder+"navaid")
			TheRaster = TheRaster + arcpy.Raster(Folder+"pipeline")
			TheRaster = TheRaster + arcpy.Raster(Folder+"protected")
			TheRaster = TheRaster + arcpy.Raster(Folder+"rec_boating")
			TheRaster = TheRaster + arcpy.Raster(Folder+"rec_fishing")
			TheRaster = TheRaster + arcpy.Raster(Folder+"rec_other")
			TheRaster = TheRaster + arcpy.Raster(Folder+"rec_wildlife")
			TheRaster = TheRaster + arcpy.Raster(Folder+"research")
			TheRaster = TheRaster + arcpy.Raster(Folder+"tribal")
			TheRaster = TheRaster + arcpy.Raster(Folder+"wrecks")
			return(TheRaster)
		except Exception, err:
			raise RuntimeError("** Error: Weighted Overlay Final Map Algebra Failed ("+str(err)+")")
	
	#############################################################
	# Multiplication Map algebra (code modified from:
	# http://forums.arcgis.com/threads/27808-Calculate-sum-of-2334-raster-layers)
	#############################################################
	def MultMapAlgebra(self, Input, weight):
		try:
			from arcpy.sa import *
			intweight = int(weight)
			TheRaster = arcpy.Raster(Input)
			TheRaster = TheRaster * intweight
			return(TheRaster)
		except Exception, err:
			raise RuntimeError("** Error: Multiplication Map Algebra Failed ("+str(err)+")")

	#############################################################
	# Convert values greater than 0 in the output raster to 1
	#############################################################
	def Reclass(self,Input):
		try:
			from arcpy.sa import *
			TheRaster = Reclassify(Input, "Value", RemapRange([[1,25,1]]))
			return(TheRaster)
		except Exception, err:
			raise RuntimeError("** Error: Reclass failed: ("+str(err)+")")

#####################################################################
# Class to interface with CONVERSION TOOLS toolbox
#####################################################################
class ConInterface:
	
	#############################################################
	# Convert a polygon to a raster (Conversion Tools -> To
	# Raster -> Polygon to Raster)
	#############################################################
	def PolygonToRaster(self,Input,Output,SnapRas):
		try:
			# First set environment settings to include
			# a snapraster so all rasters line up, then
			# run the conversion tool
			arcpy.env.snapRaster = SnapRas
			arcpy.env.extent = "-129.163686 41.997525 -123.363686 46.430858"
			arcpy.PolygonToRaster_conversion(Input,"RasVal",Output,"MAXIMUM_AREA","RasVal","1.66666666666667E-02")

		except Exception, err:
			raise RuntimeError("**Error: Polygon to Raster Failed("+str(err)+")")

	#############################################################
	# Convert a polyline to a raster (Conversion Tools -> To
	# Raster -> Polyline to Raster)
	#############################################################
	def PolylineToRaster(self,Input,Output,SnapRas):
		try:
			# First set environment settings to include
			# a snapraster so all rasters line up, then
			# run the conversion tool
			arcpy.env.snapRaster = SnapRas
			arcpy.env.extent = "-129.163686 41.997525 -123.363686 46.430858"
			arcpy.PolylineToRaster_conversion(Input,"RasVal",Output,"MAXIMUM_COMBINED_LENGTH","NONE","1.66666666666667E-02")
		except Exception, err:
			raise RuntimeError("**Error: Polyline to Raster Failed("+str(err)+")")

	#############################################################
	# Convert a point to a raster (Conversion Tools -> To 
	# Raster -> Point to Raster)
	#############################################################
	def PointToRaster(self,Input,Output,SnapRas):
		try:
			# First set environment settings to include
			# a snapraster so all rasters line up, then
			# run the conversion tool
			arcpy.env.snapRaster = SnapRas
			arcpy.env.extent = "-129.163686 41.997525 -123.363686 46.430858"
			arcpy.PointToRaster_conversion(Input,"RasVal",Output,"MOST_FREQUENT","NONE","1.66666666666667E-02")			
		except Exception, err:
			raise RuntimeError("**Error: Point to Raster Failed("+str(err)+")")	

#####################################################################
# Class to interface with DATA MANAGEMENT TOOLS toolbox
#####################################################################
class DMInterface:
	
	#############################################################
	# Add a field to an attribute table (Data Management Tools
	# -> Fields -> Add Field)
	#############################################################
	def AddField(self,Input):
		try:
			arcpy.AddField_management(Input,"RasVal","SHORT","#","#","#","#","NON_NULLABLE","NON_REQUIRED","#")
		except Exception, err:
			raise RuntimeError("**Error: Add Field Failed("+str(err)+")")
		
	#############################################################
	# Calculate field in an attribute table (Data Management 
	# Tools -> Fields -> Calculate Field)
	#############################################################
	def CalculateField(self,Input,Field):
		try:
			arcpy.CalculateField_management(Input,Field,"1","VB","#")
		except Exception, err:
			raise RuntimeError("**Error: Calculate Field Failed("+str(err)+")")
		
	#############################################################
	# Define projection as GCS North American 1983 (Data
	# Management Tools -> Projections and Transformations -> 
	# Define Projection)
	#############################################################
	def DefineProj(self, Input):
		try:
			arcpy.DefineProjection_management(Input,"GEOGCS['GCS_North_American_1983',DATUM['D_North_American_1983',SPHEROID['GRS_1980',6378137.0,298.257222101]],PRIMEM['Greenwich',0.0],UNIT['Degree',0.0174532925199433]]")
		except Exception, err:
			raise RuntimeError("**Error: Define Projection Failed("+str(err)+")")

	#############################################################
	# Copy Raster (Data Management Tools -> Raster -> Raster
	# Dataset -> Copy Raster)
	#############################################################
	def CopyRaster(self, Input, OutputFolder, ExtentRas):
		try:
			from arcpy.sa import *
			ExtentFile = Raster(ExtentRas)
			ExtentCoords = ExtentFile.extent
			arcpy.env.extent = ExtentCoords
			arcpy.env.snapRaster = ExtentRas
			arcpy.CopyRaster_management(Input,OutputFolder+"CR"+Input,"#","#","#","NONE","NONE","#")
		except Exception, err:
			raise RuntimeError("**Error: Copy Raster Failed("+str(err)+")")
		
	#############################################################
	# Copy Shapefile (Data Management Tools -> Features -> Copy 
	# Features)
	#############################################################
	def CopyShapefile(self, Input, Output):
		try:
			arcpy.CopyFeatures_management(Input, Output)
		except Exception, err:
			raise RuntimeError("**Error: Copy Shapefile Failed("+str(err)+")")		
		

#####################################################################
# Class to interface with ANALYSIS TOOLS toolbox
#####################################################################
class ATInterface:

	#############################################################
	# Clip a shapefile using the Oregon Waters polygon
	#############################################################
	def Clip(self, Input, Output):
		try:	
			arcpy.Clip_analysis(Input,"Z:/MMS/Sullivan/Thesis/Data/Parameters/OR_Waters_ForClipping.shp",Output,"#")
		except Exception, err:
			raise RuntimeError("**Error: Clip Shapefile Failed("+str(err)+")")

	#############################################################
	# Buffer a shapefile
	#############################################################
	def Buffer(self, Input, Output, BufferDist):
		try:	
			arcpy.Buffer_analysis(Input,Output,BufferDist,"FULL","ROUND","ALL","#")
		except Exception, err:
			raise RuntimeError("**Error: Buffer Shapefile Failed("+str(err)+")")	
		
#####################################################################
# Class to interface with ARCPY CLASSES
#####################################################################
class ArcPyClasses:

	#############################################################
	# Get the spatial extent of a shapefile
	#############################################################
	def GetExtentShp(self, Input):
		try:
			desc = arcpy.Describe(Input)
			return (desc.extent.XMin, desc.extent.YMin, desc.extent.XMax, desc.extent.YMax)
		except Exception, err:
			raise RuntimeError("**Error: Get Shapefile Extent Failed("+str(err)+")")
		
	#############################################################
	# Get the spatial extent of a raster
	#############################################################
	def GetExtent(self, Input):
		try:
			# Specify the raster to be evaluated
			RasFile = arcpy.Raster(Input)
			
			# Read the extent property of the raster and
			# return it as the function output
			RasExtent = RasFile.extent
			return (RasExtent.YMax, RasExtent.YMin, RasExtent.XMin, RasExtent.XMax)
		except Exception, err:
			raise RuntimeError("**Error: Get Raster Extent Failed("+str(err)+")")

	#############################################################
	# Get the arguments from the user inputs in the toolbox
	#############################################################
	def GetArguments(self, Boolean):
		try:
			if (True):
				TheFolder = arcpy.GetParameterAsText(0)
				TheOutputFolder = arcpy.GetParameterAsText(1)
				return (TheFolder, TheOutputFolder)
		except Exception, err:
			raise RuntimeError("**Error: Get Arguments Failed("+str(err)+")")
	
	#############################################################
	# Get a list of rasters in a folder using a wildcard
	#############################################################
	def GetRasterList(self, Folder, WildCard):
		try:
			arcpy.env.workspace = Folder
			rasterlist = arcpy.ListRasters(WildCard, "")
			return rasterlist
		except Exception, err:
			raise RuntimeError("**Error: Get Raster list Failed("+str(err)+")")
		
	#############################################################
	# Get the shape of a feature
	#############################################################
	def GetShape(self, Input):
		try:
			desc = arcpy.Describe(Input)
			shape = desc.shapeType
			return shape
		except Exception, err:
			raise RuntimeError("**Error: Get Shape Failed ("+str(err)+")")
	
# For reference, this is how the constant value snap raster was made "fullraster" (manually in Arc, exported snippet)
# Used 'Oregon waters for clipping' and added to top and right in order to perfectly fit 266 x 348 (see spreadsheet for calculation)
# arcpy.CreateConstantRaster_sa("Z:/MMS/Sullivan/Thesis/Data/Parameters/fullraster","0","INTEGER","1.66666666666667E-02","-129.163686 41.997525 -123.363686 46.430858")