#####################################################################
#   Prepare category rasters for the uncertainty analysis, using the BUFFERED input files
#
#   Batch processing of shapefiles organized in folders, to obtain a single raster for
#   each folder, which shows total presence of shapefiles in each cell (value of 1 for
#   presence, 0 for absence)
#
#   By: Colleen Sullivan
#   Date: 6/2012
#
#####################################################################

# All of the code occurs in a try/except block to help catch errors
try:
     print("Start")
          
     # Main folder in a variable to shorten file paths to type later...
     TopLevelFolder = "Z:/MMS/Sullivan/Thesis/Data/BufferedRasters/"
     
     # A log is kept of processing progress, any errors returned will be saved to this 
     # log, first the start time of processing is
     # recorded:
     TheTextFile=open("Z:/MMS/Sullivan/Thesis/Data/BatchShpToRasLog_Buffered.txt","w")
     import datetime
     now = datetime.datetime.now()
     timestamp = format(now.month) + "/" + format(now.day) + "/" + format(now.year) + " " + format(now.hour) + ":" + format(now.minute) + ":" + format(now.second)
     TheTextFile.write("Beginning processing: " + timestamp + "\n")
     
     # A counter is used to guarantee unique intermediate file names, here the value is
     # set to 0
     counter = 0
     
     # Import the module containing the ArcGIS Interface classes
     import ArcGISInterfaceModule
     
     # Import the operating system module to allow file/folder creation and loops later
     import os
     
     # Remove results of the last run before beginning, if they exist, and either way 
     # create folders to store results of this run
     import shutil
     
     if os.path.exists(TopLevelFolder + "BatchShpToRasTemp/"):
          shutil.rmtree(TopLevelFolder + "BatchShpToRasTemp/")

     if os.path.exists(TopLevelFolder + "Clipped/"):
          shutil.rmtree(TopLevelFolder + "Clipped/")
     os.makedirs(TopLevelFolder + "Clipped/")

     if os.path.exists(TopLevelFolder+"Rasters/"):
          shutil.rmtree(TopLevelFolder+"Rasters/")
     os.makedirs(TopLevelFolder+"Rasters/")
     
     # To batch process, the code must recongize all of the subfolders of the top level 
     # folder (each is its own data category). So, a list of folders is generated and a 
     # loop is used to run the code for each shapefile found in each folder. This code 
     # was adapted from that on Jim Graham's course website: 
     # http://ibis.colostate.edu/DH.php?WC=/WS/Jim/Geo599/08_5_BatchProcessinWithArcGIS.html
     TheList=os.listdir(TopLevelFolder+"Input_Buff/")
     for TheFolder in TheList:

          # Create a temporary folder to store intermediate files, the folder will be
          # deleted as the final step of the loop
          TempScratchFolder = TopLevelFolder + "BatchShpToRasTemp/"
          os.makedirs(TempScratchFolder)
          
          # Create a subfolder in 'Clipped' for this category to store the clipped files
          ClippedPath = TopLevelFolder+"Clipped/"+TheFolder+"/"
          os.makedirs(ClippedPath)          
          
          # Set the processing path
          ProcessingPath = TopLevelFolder+"Input_Buff/"+TheFolder+"/"
          
          TheTextFile.write("\nNow processing shapefiles in path:" + ProcessingPath + "\n")
          print("Now processing shapefiles in path:" + ProcessingPath + "\n")
          
          TheProcessingList=os.listdir(ProcessingPath)
          for TheFile in TheProcessingList:
               
               # The file name is split from its extension for use in the commands that
               # follow and to determine if it is a .shp, each shapefile name appears
               # multiple times in the list because of its component files, so this 
               # ensures each shapefile is only actually processed once.
               TheFileName, TheFileExtension = os.path.splitext(TheFile)
               if (TheFileExtension==".shp"):
                    TheTextFile.write("  - Shapefile: " + TheFileName + " -\n")
                    
                    # ~~ The Analysis Tools class is accessed ~~
                    TheInterface=ArcGISInterfaceModule.ATInterface()
                    
                    # The shapefile is clipped to Oregon State Waters and saved to a
                    # separate folder to preseve the input data in case it needs to be 
                    # re-processed later
                    TheInterface.Clip(ProcessingPath+TheFile, ClippedPath+TheFile)
                    TheTextFile.write("    Done with 1 - Clip to new folder\n")

                    # ~~ The Data Management Tools class is accessed ~~
                    TheInterface=ArcGISInterfaceModule.DMInterface()
                    
                    # A field called 'RasVal' is added to the shapefile so it can later 
                    # be used in MapAlgebra
                    TheInterface.AddField(ClippedPath+TheFile)
                    TheTextFile.write("    Done with 2 - Add Field\n")
                    
                    # The 'RasVal' field is given a value of 1 so that the converted 
                    # raster has a value of 1 wherever it is present
                    TheInterface.CalculateField(ClippedPath+TheFile,"RasVal")
                    TheTextFile.write("    Done with 3 - Calculate Field\n")
                                                         
                    # ~~ The ArcPy class is accessed ~~
                    TheInterface = ArcGISInterfaceModule.ArcPyClasses()
     
                    # The shapefile type (point, polyline, polygon) is determined to 
                    # allow proper conversion to raster
                    shape = TheInterface.GetShape(ClippedPath+TheFile)
                    
                    # ~~ The Conversion Tools class is accessed ~~
                    TheInterface=ArcGISInterfaceModule.ConInterface()
                    
                    # The shapefile (shape now known) is converted to a raster using a
                    # name derived from its original name and a counter, shortened because
                    # the names must ultimately be no more than 13 characters, a snap 
                    # raster and environment settings are used to ensure that the many 
                    # outputs line up with one another for proper map algebra.             
                    OutName = "r" + format(counter) + TheFile[0:6]
                    
                    if shape == "Polygon":
                         TheInterface.PolygonToRaster(ClippedPath+TheFile, TempScratchFolder+OutName, "Z:/MMS/Sullivan/Thesis/Data/Parameters/fullraster")
                    elif shape == "Polyline":
                         TheInterface.PolylineToRaster(ClippedPath+TheFile, TempScratchFolder+OutName, "Z:/MMS/Sullivan/Thesis/Data/Parameters/fullraster")
                    else:
                         TheInterface.PointToRaster(ClippedPath+TheFile, TempScratchFolder+OutName, "Z:/MMS/Sullivan/Thesis/Data/Parameters/fullraster")
                    TheTextFile.write("    Done with 4 - Convert feature to raster\n")
 
                    # Counter is increased so the next shapefile processed has a unique
                    # name
                    counter = counter + 1
                    
          TheTextFile.write("\n  - All shapefiles of this category have completed individual processing. Beginning Map Algebra steps. -\n")
          
          # In order to calculate cell statistics on our new rasters, they cannot have
          # NoData values
          
          # ~~ The ArcPy class is accessed ~~
          TheInterface = ArcGISInterfaceModule.ArcPyClasses()
          
          # The list of rasters in the processed folder is obtained
          TheList = TheInterface.GetRasterList(TempScratchFolder,"*")
          
          # ~~ The Spatial Analyst Tools class is accessed ~~
          TheInterface = ArcGISInterfaceModule.SAInterface()
          
          # The list of copied rasters is looped through to convert NoData to 0
          for TheFile in TheList:
               temp = TheInterface.ConvertNoData(TheFile)
               temp.save(TempScratchFolder+"z"+TheFile)
          TheTextFile.write("    Done with 5 - All rasters processed to change NoData to 0 values\n")
          
          # Our raster layers of a single category are now ready for cell statistics to
          # form a single category raster, the next step adds them together
          
          # ~~ The ArcPy class is accessed ~~
          TheInterface=ArcGISInterfaceModule.ArcPyClasses()
          
          # The list of rasters without NoData values in the processed folder is obtained
          # (using a wildcard to limit the search)
          TheList = TheInterface.GetRasterList(TempScratchFolder,"z*")
          
          # ~~ The Spatial Analyst Tools class is accessed ~~
          TheInterface = ArcGISInterfaceModule.SAInterface()
          
          # An expression is generated and evaluated to add the rasters together
          temp = TheInterface.MapAlgebra(TheList)
          temp.save(TempScratchFolder+TheFolder)
          TheTextFile.write("    Done with 6 - Map algebra complete, final raster generated\n")
          
          # Currently the output shows the number of shapefiles that occur in each cell,
          # it needs to show only use or non-use (because the values of use, 1, will later
          # be multiplied by the weight for the category, and then added to the other 
          # categories) so the 'Value' field is recalculated to change numbers greater 
          # than 0 to 1
          temp = TheInterface.Reclass(TempScratchFolder+TheFolder)
          temp.save(TopLevelFolder+"Rasters/"+TheFolder)          
          TheTextFile.write("    Done with 7 - Reclassify Raster\n")
          
          # ~~ The Data Management Tools class is accessed ~~
          TheInterface = ArcGISInterfaceModule.DMInterface()
          
          # The projection of the created raster is defined so that it matches the 
          # shapefiles processed
          TheInterface.DefineProj(TopLevelFolder+"Rasters/"+TheFolder)
          TheTextFile.write("    Done with 8 - Define projection of raster\n")
          
          # The folder of intermediate files is deleted
          shutil.rmtree(TempScratchFolder)
          TheTextFile.write("    Done with 9 - Intermediate files for this category deleted\n\n")

# In case of error, the code will break to here and return the error message to the 
# text file
except Exception, err:
     print("*** Error. Stopped because: " + str(err))
     TheTextFile.write(str(err)+"\n")

# A final message is printed with the end processing time and the text file log is closed.
now = datetime.datetime.now()
timestamp = format(now.month) + "/" + format(now.day) + "/" + format(now.year) + " " + format(now.hour) + ":" + format(now.minute) + ":" + format(now.second)
TheTextFile.write("Completed Script: " + timestamp)
TheTextFile.close
print("End")